@@ -1,4 +1,4 @@
-from django.conf.urls.defaults import *
+from django.conf.urls.defaults import patterns, url, include
 from django.contrib import admin
 
 from django_roa_client.views import home
@@ -6,6 +6,6 @@ from django_roa_client.views import home
 admin.autodiscover()
 
 urlpatterns = patterns('',
-    url(r'^admin/(.*)', admin.site.root),
+    url(r'^admin/', include(admin.site.urls)),
     url(r'^$', home),
 )
