@@ -1,13 +1,9 @@
 import logging
 
-from django.conf import settings
 from django.contrib.auth.models import User, Message, Group, Permission
-from django.core.exceptions import ObjectDoesNotExist
-from django.core import serializers
 from django.db import models
-from django.http import Http404, HttpResponse, HttpResponseNotAllowed
+from django.http import Http404
 from django.shortcuts import get_object_or_404, _get_queryset
-from django.utils.encoding import smart_unicode
 
 from piston.handler import BaseHandler, AnonymousBaseHandler
 from piston.utils import rc
@@ -15,7 +11,8 @@ from piston.utils import rc
 from django_roa_server.models import RemotePage, RemotePageWithManyFields, \
     RemotePageWithBooleanFields, RemotePageWithCustomSlug, \
     RemotePageWithOverriddenUrls, RemotePageWithRelations, \
-    RemotePageWithNamedRelations, RemotePageWithRelationsThrough
+    RemotePageWithNamedRelations, RemotePageWithRelationsThrough, \
+    RemotePageWithCustomPrimaryKey
 
 logger = logging.getLogger("django_roa_server")
 
@@ -24,26 +21,30 @@ class ROAHandler(BaseHandler):
 
     def flatten_dict(self, dct):
         return dict([ (str(k), dct.get(k)) for k in dct.keys() \
-            if (k, dct.get(k)) != (u'id', u'None')])
+            if (k, dct.get(k)) != (u'id', u'None') and (k, dct.get(k)) != (u'pk', u'None')])
 
     @staticmethod
     def _get_object(model, *args, **kwargs):
-        return get_object_or_404(model, id=kwargs['id'])
-        
+        return get_object_or_404(model, pk=kwargs['pk'])
+
     def read(self, request, *args, **kwargs):
         """
         Retrieves an object or a list of objects.
         """
         if not self.has_model():
             return rc.NOT_IMPLEMENTED
-        
+
+        logger.debug('Args: %s' % str(args))
+        logger.debug('Kwargs: %s' % str(kwargs))
+
         if kwargs.values() != [None]:
             # Returns a single object
             return [self._get_object(self.model, *args, **kwargs)]
-        
+
         # Initialization
         queryset = _get_queryset(self.model)
-        
+        logger.debug('Before filters: %s' % str(queryset))
+
         # Filtering
         filters, excludes = {}, {}
         for k, v in request.GET.iteritems():
@@ -52,39 +53,42 @@ class ROAHandler(BaseHandler):
             if k.startswith('exclude_'):
                 excludes[k[8:]] = v
         queryset = queryset.filter(*filters.items()).exclude(*excludes.items())
-        
+
+        logger.debug('Filters: %s' % str(filters))
+        logger.debug('Excludes: %s' % str(excludes))
+        logger.debug('After filters: %s' % str(queryset))
+
         # Ordering (test custom parameters' name)
         if 'order' in request.GET:
             order_bys = request.GET['order'].split(',')
             queryset = queryset.order_by(*order_bys)
-        
+
         # Slicing
         limit_start = int(request.GET.get('limit_start', 0))
         limit_stop = request.GET.get('limit_stop', False) and int(request.GET['limit_stop']) or None
         queryset = queryset[limit_start:limit_stop]
-        
+
         obj_list = list(queryset)
         if not obj_list:
             raise Http404('No %s matches the given query.' % queryset.model._meta.object_name)
         logger.debug(u'Objects: %s retrieved' % [unicode(obj) for obj in obj_list])
         return queryset
-        
+
     def create(self, request, *args, **kwargs):
         """
         Creates an object given request args, returned as a list.
         """
         if not self.has_model():
             return rc.NOT_IMPLEMENTED
-        
+
         data = request.POST.copy()
-        keys = data.keys()
-        
+
         values = {}
         for field in self.model._meta.local_fields:
             field_value = data.get(field.name, None)
-            
+
             if field_value not in (u'', u'None'):
-                
+
                 # Handle FK fields
                 if field.rel and isinstance(field.rel, models.ManyToOneRel):
                     field_value = data.get(field.attname, None)
@@ -92,7 +96,7 @@ class ROAHandler(BaseHandler):
                         values[field.attname] = field.rel.to._meta.get_field(field.rel.field_name).to_python(field_value)
                     else:
                         values[field.attname] = None
-                
+
                 # Handle all other fields
                 else:
                     if isinstance(field, models.fields.BooleanField):
@@ -102,11 +106,10 @@ class ROAHandler(BaseHandler):
                             field_value = float(field_value)
                     values[field.name] = field_value
 
-        object = self.model.objects.create(**values)
-        
-        response = [self.model.objects.get(id=object.id)]
-        #response = [object]
-        logger.debug(u'Object "%s" created' % unicode(object))
+        obj = self.model.objects.create(**values)
+
+        response = [self.model.objects.get(pk=obj.pk)]
+        logger.debug(u'Object "%s" created' % unicode(obj))
         return response
 
     def update(self, request, *args, **kwargs):
@@ -115,14 +118,14 @@ class ROAHandler(BaseHandler):
         """
         if not self.has_model():
             return rc.NOT_IMPLEMENTED
-        
+
         data = request.PUT.copy()
         logger.debug(u'Received: %s as PUT data' % data)
-        object = self._get_object(self.model, *args, **kwargs)
-        
+        obj = self._get_object(self.model, *args, **kwargs)
+
         for field in self.model._meta.local_fields:
             field_name = field.name
-            
+
             # Handle FK fields
             if field.rel and isinstance(field.rel, models.ManyToOneRel):
                 field_value = data.get(field.attname, None)
@@ -130,8 +133,8 @@ class ROAHandler(BaseHandler):
                     field_value = field.rel.to._meta.get_field(field.rel.field_name).to_python(field_value)
                 else:
                     field_value = None
-                setattr(object, field.attname, field_value)
-            
+                setattr(obj, field.attname, field_value)
+
             # Handle all other fields
             elif field_name in data:
                 field_value = data[field_name]
@@ -147,14 +150,13 @@ class ROAHandler(BaseHandler):
                 elif isinstance(field, models.fields.CharField):
                     if field_value is None:
                         field_value = u''
-                setattr(object, field_name, field_value)
-        
-        object.save()
-        
-        response = [self.model.objects.get(id=object.id)]
-        #response = [object]
+                setattr(obj, field_name, field_value)
+
+        obj.save()
+
+        response = [self.model.objects.get(pk=obj.pk)]
         logger.debug(u'Object "%s" modified with %s' % (
-            unicode(object), unicode(data.items())))
+            unicode(obj), unicode(data.items())))
         return response
 
     def delete(self, request, *args, **kwargs):
@@ -163,13 +165,13 @@ class ROAHandler(BaseHandler):
         """
         if not self.has_model():
             raise NotImplementedError
-        
+
         try:
-            object = self._get_object(self.model, *args, **kwargs)
-            object.delete()
+            obj = self._get_object(self.model, *args, **kwargs)
+            obj.delete()
             logger.debug(u'Object "%s" deleted, remains %s' % (
-                unicode(object), 
-                [unicode(object) for object in self.model.objects.all()]))
+                unicode(obj),
+                [unicode(obj) for obj in self.model.objects.all()]))
 
             return rc.DELETED
         except self.model.MultipleObjectsReturned:
@@ -187,10 +189,10 @@ class ROACountHandler(BaseHandler):
         """
         if not self.has_model():
             return rc.NOT_IMPLEMENTED
-        
+
         # Initialization
         queryset = _get_queryset(self.model)
-        
+
         # Filtering
         filters, excludes = {}, {}
         for k, v in request.GET.iteritems():
@@ -199,12 +201,12 @@ class ROACountHandler(BaseHandler):
             if k.startswith('exclude_'):
                 excludes[k[8:]] = v
         queryset = queryset.filter(*filters.items()).exclude(*excludes.items())
-        
+
         # Ordering
         if 'order_by' in request.GET:
             order_bys = request.GET['order_by'].split(',')
             queryset = queryset.order_by(*order_bys)
-        
+
         # Counting
         counter = queryset.count()
         logger.debug(u'Count: %s objects' % counter)
@@ -212,17 +214,17 @@ class ROACountHandler(BaseHandler):
 
 
 class ROAWithSlugHandler(ROAHandler):
-    
+
     @staticmethod
     def _get_object(model, *args, **kwargs):
         """Returns an object from a slug.
-        
+
         Useful when the slug is a combination of many fields.
         """
-        id, slug = kwargs['object_slug'].split('-', 1)
-        object = get_object_or_404(model, id=id, slug=slug)
-        return object
-    
+        pk, slug = kwargs['object_slug'].split('-', 1)
+        obj = get_object_or_404(model, pk=pk)
+        return obj
+
 
 class RemotePageHandler(ROAHandler):
     model = RemotePage
@@ -251,7 +253,19 @@ class RemotePageWithCustomSlugHandler(ROAWithSlugHandler):
 class RemotePageWithCustomSlugCountHandler(ROACountHandler):
     model = RemotePageWithCustomSlug
 
-    
+class RemotePageWithCustomPrimaryKeyHandler(ROAHandler):
+    model = RemotePageWithCustomPrimaryKey
+
+class RemotePageWithCustomPrimaryKeyCountHandler(ROACountHandler):
+    model = RemotePageWithCustomPrimaryKey
+
+class RemotePageWithCustomPrimaryKeyCount2Handler(ROACountHandler):
+    model = RemotePageWithCustomPrimaryKey
+
+    def read(self, request, *args, **kwargs):
+        return 'invalid counter'
+
+
 class RemotePageWithOverriddenUrlsHandler(ROAWithSlugHandler):
     model = RemotePageWithOverriddenUrls
 
@@ -265,21 +279,21 @@ class RemotePageWithRelationsHandler(ROAHandler):
 class RemotePageWithRelationsCountHandler(ROACountHandler):
     model = RemotePageWithRelations
 
-    
+
 class RemotePageWithRelationsThroughHandler(ROAHandler):
     model = RemotePageWithRelationsThrough
 
 class RemotePageWithRelationsThroughCountHandler(ROACountHandler):
     model = RemotePageWithRelationsThrough
 
-    
+
 class RemotePageWithNamedRelationsHandler(ROAHandler):
     model = RemotePageWithNamedRelations
 
 class RemotePageWithNamedRelationsCountHandler(ROACountHandler):
     model = RemotePageWithNamedRelations
 
-    
+
 class UserHandler(ROAHandler):
     model = User
 
@@ -288,6 +302,6 @@ class MessageHandler(ROAHandler):
 
 class PermissionHandler(ROAHandler):
     model = Permission
-    
+
 class GroupHandler(ROAHandler):
     model = Group
